Attribute Macro ruva_macro::into_command

source ·
#[into_command]
Attributes will be given in the following format not specifying any attributes will result in default attributes which are Debug and Deserialize for body and Debug and Serialize for command

Example

#[into_command(body(Debug, Deserialize), command(Debug, Serialize))]
pub struct X{}
#[into_command(command(Debug, Serialize), body(Debug, Deserialize))]
pub struct Y{}
#[into_command(body(Debug, Deserialize))]
pub struct Z{}
#[into_command(command(Debug, Serialize))]
pub struct Q{}
#[into_command]
pub struct W{}